//book.cpp.  Copyright (C) 2004 Dan Honeycutt.  You are free
//to use this software.  There is no warranty of any kind.
#include <stdlib.h>
#include "book.h"

//CBook seeds the random number generator.  Change the following
//to 0 (or call srand(1) after startup) if you want fixed random
//numbers for debugging etc.  Of course if you do this the moves
//selected from the book won't be random.
#define seed_random_number_generator 1

#if seed_random_number_generator
#include <time.h>
#endif

//default book - follows below.
extern CBookDef bk_def_book[];

//====================================================================
//GetBookMove() returns a book move for the current position.
//Calls SelectBookMove() which chooses the move.
//====================================================================
const char *CBook::GetBookMove(const CBookPos *cur_pos) {
  const char *promote = "  NBRQ";
  static char buffer[6];
  int i, move;
  char *pc = buffer;
  if ((move = SelectBookMove(cur_pos))) {
    //book move available - format to ascii
    for (i = 0; i < 2; i++) {
      *pc++ = (char) ('a' + (move & 7));
      *pc++ = (char) ('1' + ((move & 63) >> 3));
      move >>= 6;
    }
    i = move & 7;
    if ((i > 1) && (i < 6)) *pc++ = promote[i];
  }
  *pc = 0;
  return buffer;
} //GetBookMove()

//====================================================================
//SelectBookMove() makes the random selection of a move according to
//move weight.  Calls FindBookEntry() to get moves to select from.
//====================================================================
int CBook::SelectBookMove(const CBookPos *cur_pos) {
  int i, k = 0;
  char c;
  //do a quick validity check and piece count
  for (i = 0; i < 64; i++) {  
    c = cur_pos->bk_board[i];
    if (c) {
      k++;
      if ((c < 0) || (c > 15)) return 0;
    }
  }
  if (k < bk_min_men) return 0;             //too few pieces
  if (!FindBookEntry(cur_pos)) return 0;    //position not found
  if (bk_num_moves == 0) return 0;          //position has no moves

  k = 0;
  c = (char) (rand() % 100);                //0 to 99
  for (i = 0; i < bk_num_moves; i++) {
    k += bk_entry.bk_value[i];
    if (k > c) return (bk_entry.bk_moves[i]);
  }  
  return (0);   //no move
} //SelectBookMove()

//====================================================================
//FindBookEntry() looks for the current position in the book.  Returns 
//true if the position is found and member variable bk_entry will have 
//the moves.  Looks first for the current position and, if not found,
//looks again for the position with colors reversed.
//Calls BinSearch() to perform the actual search.
//====================================================================
bool CBook::FindBookEntry(const CBookPos *cur_pos) {
  int i, move, b1, b2, pro;
  char c;
  CBookPos temp;

  bk_num_moves = 0;
  bk_color_rev = 0;
  if ((i = BinSearch(GetHashKey(cur_pos)))) {
    CopyBookEntry(i);
    return 1;
  }
  //failed to find position, reverse colors and try again
  for (i = 0; i < 64; i++) {
    c = cur_pos->bk_board[i];
    if (c) c ^= 8;                    //toggle piece color
    temp.bk_board[Flip(i)] = c;      //flip square
  }
  c = cur_pos->bk_ep_target;
  if ((c < 16) || (c > 47)) c = 0;
  if (c) c = (char) Flip(c);
  temp.bk_ep_target = c;
  temp.bk_color = !cur_pos->bk_color;
  i = BinSearch(GetHashKey(&temp));
  if (!i) return 0;
  //found the color reverse position. moves have to be reversed
  bk_color_rev = 1;
  CopyBookEntry(i);
  for (i = 0; i < bk_num_moves; i++) {
    move = bk_entry.bk_moves[i];
    b1 = Flip(move & 63);
    b2 = Flip((move >> 6) & 63);
    pro = (move >> 12) & 7;
    bk_entry.bk_moves[i] = (short)(b1 + (b2 << 6) + (pro << 12));
  }
  return 1;
} //FindBookEntry()

//====================================================================
//BinSearch() performs search for target position.  Since the keys 
//are random, the value of target tells approximately where it
//resides in the list.  BinSearch() finds a window around target then
//performs a binary search of the window.
//====================================================================
int CBook::BinSearch(bk_bbd target) {
  int lo, mid, hi;
  bk_bbd key;

  //set a window around the calculated position of target
  mid = 1 + (bk_num_pos >> 8);  //window ~ 0.5% of total positions
  hi = (int) (target >> 33);    //whittle down to signed 4 byte size
  hi = mid + (int) (bk_num_pos * (((float)hi)/2147483647.0f));  
  if (hi > bk_num_pos) hi = bk_num_pos;
  lo = hi - 2 * mid;
  if (lo < 1) lo = 1;
  //adjust window if we missed the target
  while (BookEntryKey(lo) > target) {   //window too high
    hi = lo;          //move hi down to lo
    mid *= 2;         //expand window size
    lo -= mid;        //move lo down by window size
    if (lo < 1) {     //check limits
      lo = 1;
      break;
    }
  }
  while (BookEntryKey(hi) < target) {   //window too low
    lo = hi;
    mid *= 2;
    hi += mid;
    if (hi > bk_num_pos) {
      hi = bk_num_pos;
      break;
    }
  }
  //now do a binary search on the window
  while (lo <= hi) {
    mid = (lo + hi) / 2;
    key = BookEntryKey(mid);
    if (target < key) 
      hi = mid - 1;
    else if (target > key)
      lo = mid + 1;
    else
      return mid;       //found the position
  }
  return 0;             //search fails
} //BinSearch()

//====================================================================
//Utility Routines:
//Flip() color reverses a square
//GetHashKey() returns hash key for a CBookPos
//BookEntryKey() returns the key corresponding to index passed
//CopyBookEntry() copies entry to bk_entry and sets number of moves
//====================================================================
int CBook::Flip(int sq) {
  return 8*(7-(sq >> 3)) + (sq & 7);
} //Flip()

bk_bbd CBook::GetHashKey(const CBookPos *p1) {
  int i;
  char c;
  bk_bbd key = 0;
  for (i = 0; i < 64; i++) {
    if ((c = p1->bk_board[i])) key ^= bk_rnd[c][i];
  }
  //bk_rnd[8][0] is black-to-move
  if (p1->bk_color) key ^= bk_rnd[8][0];
  //bk_rnd[0][sq] is en passant target
  c = p1->bk_ep_target;
  if ((c < 16) || (c > 47)) c = 0;
  if (c) key ^= bk_rnd[0][c];
  return key;
} //GetHashKey()

bk_bbd CBook::BookEntryKey(int ix) {
  if (bk_using_default) {
    return bk_def_book[ix].bk_key;
  }
  //future - file
  return 0;
} //BookEntryKey()

void CBook::CopyBookEntry(int ix) {
  int i;
  if (bk_using_default) {
    bk_entry.bk_key = bk_def_book[ix].bk_key;
    for (i = 0; i < BK_MAX_MOVES; i++) {
      bk_entry.bk_moves[i] = bk_def_book[ix].bk_moves[i];
      bk_entry.bk_value[i] = bk_def_book[ix].bk_value[i];
    }
  } else {
    //future - file
    bk_entry.bk_moves[0] = 0;   //no moves
  }
  //set the number of moves
  for (i = 0; i < BK_MAX_MOVES; i++) {
    if (!bk_entry.bk_moves[i]) break;
  }
  bk_num_moves = i;
} //CopyBookEntry()

//====================================================================
//InitBook() is called at startup by constructor.  All it does is set
//the default book.  Future plans are too look for a book on disk
//and set a file pointer to that if found.
//====================================================================
void CBook::InitBook() {
//seed random numbers unless told not to
#if seed_random_number_generator
  srand((unsigned)time(NULL));
#endif
  extern bk_bbd random_numbers[];
  //cast random_numbers[1024] to bk_rnd[16][64]
  bk_rnd = (bk_bbd (*)[64]) random_numbers;
  //set default book
  bk_using_default = 1;
  //first record has number positions and minimum pieces
  bk_num_pos = (int) bk_def_book[0].bk_key;
  bk_min_men = bk_def_book[0].bk_moves[0];
  //future - file
} //InitBook()

//====================================================================
//the book
//====================================================================
CBookDef bk_def_book[924] = {
{923,{0},{25}},
{0x00004cd1156e0a42,{63,37},{2434,1292}},
{0x0012df17cf27ef9e,{90},{64}},
{0x003af1ebb780bc67,{100},{2284}},
{0x006b3a53773c00e6,{40,38,22},{3194,2618,1661}},
{0x009789cbe7b14273,{33,32,18,17},{2356,2942,2745,2226}},
{0x00c174c387640d0d,{51,49},{3389,1699}},
{0x00e2602d2ad84abc,{100},{1350}},
{0x01a97858cb4a23d6,{63,37},{2291,2942}},
{0x021ad8b866a45bec,{100},{2267}},
{0x0280570256b1f676,{90},{1739}},
{0x029814ccb093d89a,{63,37},{2426,1978}},
{0x02c4ddb841ac7596,{71,29},{2745,2933}},
{0x02c6777874b0b731,{100},{1827}},
{0x02e29ee4ab343dc9,{100},{1112}},
{0x0329638b8895dfe2,{90},{2942}},
{0x03309304b8ce33fc,{90},{388}},
{0x0338b47e5f6c05a2,{63,37},{1539,901}},
{0x033b3bfa5c1198c8,{63,37},{2998,2942}},
{0x034077a56edea074,{100},{2737}},
{0x037f21863ddf88eb,{100},{2268}},
{0x038d1125fc5be54b,{90},{1114}},
{0x03b16117856bb2bc,{100},{1292}},
{0x03d28794a0b871bf,{100},{2291}},
{0x03f71ce16ae7962e,{100},{2331}},
{0x040050873d704eab,{90},{2551}},
{0x0429eb10f0000334,{100},{1282}},
{0x043e499372f988d5,{71,29},{4028,2608}},
{0x04a9dbf56c157ea6,{100},{2745}},
{0x0543ce24774a77f2,{100},{1253}},
{0x054900a5c4aabacd,{90},{2942}},
{0x059ad276bafb310b,{100},{2942}},
{0x05e1082682de92c9,{47,28,25},{2745,3517,2942}},
{0x060e39492d2c0bb1,{47,28,25},{2868,2738,1699}},
{0x06548ebb7023651e,{71,29},{1812,388}},
{0x06602fff59516f6d,{63,37},{1739,1422}},
{0x0672c53eb50ae3e4,{27,26,15,14,12,6},{2738,2868,2998,2426,2226,1978}},
{0x06943ca04a1dd63b,{90},{1422}},
{0x06b3fd2c2700e4ab,{33,32,18,17},{1292,1674,2434,1858}},
{0x06e8f834bdc085dd,{100},{2266}},
{0x071ac9a5f924cd3f,{47,28,25},{2226,2291,2356}},
{0x075f7b4b1957c637,{100},{2673}},
{0x07bb701e3028e01b,{90},{4028}},
{0x0959450f9df5fca5,{100},{1153}},
{0x0961e2d691b3184e,{27,26,24,23},{2291,2356,2942,2226}},
{0x0a323b3c3d0ea4a5,{47,28,25},{2426,732,2226}},
{0x0a52491f9353477a,{51,30,19},{2942,2868,2738}},
{0x0a706a55000e2905,{51,30,19},{2942,2868,2998}},
{0x0a96142f7c247fdb,{100},{1162}},
{0x0a988180f49c8469,{100},{2803}},
{0x0ae98f287a90ee47,{63,37},{3321,3063}},
{0x0b42622c90ec287f,{100},{2299}},
{0x0b84dcf0f9815701,{100},{2161}},
{0x0b8c498f4f6ff954,{100},{4028}},
{0x0ba1308bce3c04aa,{43,41,16},{1292,1350,2266}},
{0x0bbc366f060d4ed2,{100},{1683}},
{0x0c98ab0b70fbcf33,{100},{657}},
{0x0ce07e6f75cbfe92,{63,37},{2022,1894}},
{0x0cf380140474f8c1,{100},{2325}},
{0x0de5949efff23f4f,{44,26,16,14},{3517,2356,2942,3321}},
{0x0de7080cb0ab475f,{90},{1829}},
{0x0e42816197e9fe76,{90},{1153}},
{0x0e6d4b6438ddfef2,{33,32,18,17},{2356,2942,2291,2226}},
{0x0ea52a8ffa24f4d7,{63,37},{1350,1153}},
{0x0fae367e050bf635,{63,37},{2745,2803}},
{0x0fe6cbb6e419e698,{51,49},{1739,1674}},
{0x100119752cc623e8,{100},{1153}},
{0x10fb9b77273d8463,{63,37},{1804,1350}},
{0x111e8b831b3397cc,{47,28,25},{3322,2745,3321}},
{0x1137a3065164a9bc,{90},{1153}},
{0x11652307b55967a6,{63,37},{2745,2683}},
{0x119ddc1f207970ef,{100},{2939}},
{0x12bab8b6f6e87f18,{90},{4028}},
{0x132f931e020c3bb1,{51,30,19},{1422,1350,1292}},
{0x13d2b42dbac651c3,{71,29},{2942,2803}},
{0x142d4c3571f61190,{100},{1810}},
{0x1437abdf5a821301,{100},{1762}},
{0x144d0ece40872033,{38,22,21,19},{2745,1837,2237,1661}},
{0x149f7aaf46e2394a,{90},{1739}},
{0x15bb693c0a5b97d8,{100},{3517}},
{0x1618f0d951f9bcd8,{35,33,32},{2356,2291,2226}},
{0x17469c1a1f31071a,{71,29},{1699,3063}},
{0x178412fb630a4755,{39,23,14,13,11},{1422,1674,1804,2434,1153}},
{0x17868e692c533f45,{47,28,25},{1350,1804,1153}},
{0x1818a9eaad3d5821,{51,30,19},{2868,1699,2608}},
{0x185d859159a2b729,{100},{2942}},
{0x1971aff6743ea283,{27,26,15,14,12,6},{2942,2738,2226,1978,2998,2868}},
{0x198ea4b778c09d7b,{100},{2942}},
{0x19a54c884605d3fa,{40,38,22},{1282,705,657}},
{0x19b19d63731971b5,{33,32,18,17},{1804,1739,1350,1153}},
{0x1a0df2208a6da537,{100},{1764}},
{0x1a1c12e22e0ce0e0,{38,22,21,19},{1837,2868,2291,2226}},
{0x1a5880ace9c57723,{90},{901}},
{0x1a6cdba9800f8117,{43,41,16},{3389,2237,2204}},
{0x1ade9243cce3e552,{36,34,19,11},{1153,705,2332,2268}},
{0x1af4933c34104279,{63,37},{1350,1804}},
{0x1ba4abe6a6a14c8d,{100},{801}},
{0x1bb7821ad08a0ad7,{90},{705}},
{0x1cbef16d80dc807d,{90},{1422}},
{0x1ceb3151dbab99ca,{90},{901}},
{0x1d01031aa629e480,{90},{1749}},
{0x1d3aafcfb5131008,{63,37},{2434,786}},
{0x1d5e2a3d28634416,{38,22,21,19},{1669,1869,1422,1350}},
{0x1d654e073fb6f2de,{51,30,19},{1674,901,1739}},
{0x1db0ba599ef8caa3,{100},{2022}},
{0x1e4c615da24d277e,{32,19,18,16,15},{2745,2998,2291,2942,2868}},
{0x1e50fc08b77e1895,{51,30,19},{2356,2291,2803}},
{0x1e54de8fa242fc66,{100},{1161}},
{0x1e770567b59891b6,{47,28,25},{2942,2745,2237}},
{0x1f117e6800de0c0f,{63,37},{1153,901}},
{0x1f12f1ec03a39165,{71,29},{2942,3321}},
{0x1f24eb19465057ce,{100},{1436}},
{0x1f3b48208bb534b3,{47,28,25},{2226,3063,2291}},
{0x1fc21e0b46504c6a,{63,37},{2942,1699}},
{0x20431527dd3d7b1d,{71,29},{1350,1221}},
{0x20508745ad2611f6,{100},{2268}},
{0x209dedc4d9604594,{63,37},{2226,2291}},
{0x20b1ae8615590974,{51,30,19},{388,1812,1032}},
{0x210a79e4767b1ae4,{63,37},{2942,2426}},
{0x213431aa33f94c01,{100},{1746}},
{0x21b08e943b4a7077,{100},{1153}},
{0x21eb0ec1fc8bef59,{63,37},{2333,1804}},
{0x222356beeb80cf44,{32,19,17,16,9,7},{1350,1221,1674,705,1162,1153}},
{0x22491f2d0872ee17,{47,28,25},{2942,2745,2803}},
{0x228fbed9f7ed46df,{63,37},{774,1350}},
{0x22b1603d21639b69,{90},{1350}},
{0x22e5d8f46396b3da,{63,37},{2291,2356}},
{0x232501a769fb0958,{90},{2942}},
{0x239656f2655c1dd7,{51,49},{2942,3517}},
{0x23dce4769a963b34,{100},{4028}},
{0x2490c480cb019551,{56,23,21},{3517,2745,2803}},
{0x24aba0badcd42399,{63,37},{3517,2745}},
{0x2558603eb28c50e6,{71,29},{1032,706}},
{0x25a996a8f00da87e,{100},{1749}},
{0x25aaa4532b7f99ba,{44,26,16,14},{2434,1669,773,1282}},
{0x2608bf5e7c13f375,{100},{2426}},
{0x26ccd466677f5b3c,{63,37},{2998,2942}},
{0x27040c7c4e391b17,{40,38,22},{1674,1221,705}},
{0x271eb15cdfc36a9e,{71,29},{1749,2258}},
{0x275dbeb25842c019,{30,28,27,15},{1978,2617,2608,2745}},
{0x27cbeff3185baa9d,{71,29},{1869,786}},
{0x27d54a5b117d4258,{63,37},{1350,1669}},
{0x285d6d9ca72d8bd9,{100},{1357}},
{0x286dc4ef0015b162,{100},{2282}},
{0x29690bc9638aefa7,{27,26,15,14,12,6},{2356,2942,2226,2868,2998,2738}},
{0x29a9395c64ad3c91,{90},{901}},
{0x29d4cfac69a600bd,{43,41,16},{1739,1674,1422}},
{0x2a38f4db72dd13b2,{100},{1749}},
{0x2a402493fe713a07,{47,28,25},{1153,1739,1350}},
{0x2a67ddfcfc97b324,{51,30,19},{1153,1350,1422}},
{0x2a99b6a1230aa6bf,{63,37},{2203,2267}},
{0x2ae268173f095516,{51,30,19},{2434,1221,1162}},
{0x2aedbccae8e9f788,{100},{2942}},
{0x2afde0f6f45dd51d,{43,41,16},{2291,2942,2226}},
{0x2b01a6f349d12e9d,{63,37},{2942,2237}},
{0x2b5a3bde60968a32,{47,28,25},{1350,1669,1869}},
{0x2b97481c98d13865,{90},{3517}},
{0x2be311defb0db916,{63,37},{1292,578}},
{0x2c33449b8fefd12d,{47,28,25},{1487,1227,1674}},
{0x2c4e510bc6333bf4,{63,37},{2291,2942}},
{0x2cf80c38c4b67da8,{71,29},{4028,2738}},
{0x2cf9cab24db5888b,{100},{1828}},
{0x2da145f83a165327,{100},{2291}},
{0x2da4780af5792342,{100},{705}},
{0x2dc49a0e99ca8ac3,{90},{2745}},
{0x2de7658218171260,{40,38,22},{2434,1350,2266}},
{0x2def41f2e462f3dd,{100},{1153}},
{0x2e03574da7300af3,{47,28,25},{1762,2608,2683}},
{0x2e14f5ce25c98112,{100},{1282}},
{0x2ec66ea87999267d,{47,28,25},{2942,2291,2043}},
{0x2ece4ad885ecc7c0,{35,20,19,17,9},{2942,3389,2738,2226,2421}},
{0x2ed9afd3b3428e3f,{51,30,19},{1569,1749,1697}},
{0x2f14467245a92866,{100},{2803}},
{0x2f3d92ae10f28dae,{47,28,25},{2356,2738,1978}},
{0x2f503069109065bf,{26,25,24,13,12},{2356,2291,2226,2942,2745}},
{0x2f677e515bc8ee54,{90},{1739}},
{0x306a653e3d6c2565,{71,29},{388,771}},
{0x30b3c41aca6201d1,{63,37},{2868,2226}},
{0x30b9036e251a5c34,{40,38,22},{2325,1739,1153}},
{0x3159d83e8da5985f,{63,37},{1739,2117}},
{0x318a320e9c0fa82a,{100},{1739}},
{0x319515e87f49eee5,{100},{2931}},
{0x31cc772af129dcdb,{100},{2284}},
{0x31df7861905cc18d,{44,26,16,14},{1661,2291,1770,2237}},
{0x3224680a530358c6,{100},{388}},
{0x3317d73c4c3d1969,{63,37},{901,1487}},
{0x336e48218f4fb3c7,{100},{1380}},
{0x33902c34b8e18994,{44,26,16,14},{2745,2942,2803,2421}},
{0x33b7da5571d6394c,{90},{2226}},
{0x3401fd86681b87ef,{63,37},{1097,1162}},
{0x348a1180b3522f3b,{47,28,25},{2226,2942,2608}},
{0x34b842c9fb66bb0d,{71,29},{2434,1350}},
{0x34e18624b58af682,{100},{2608}},
{0x35837cce3862a856,{63,37},{2942,1978}},
{0x35b9a0c382450144,{100},{1739}},
{0x35edc8597595f03c,{71,29},{2267,1282}},
{0x363356685e1e3826,{90},{1999}},
{0x36af8b10dbe58b21,{100},{4028}},
{0x36d6afe33dbf95a3,{51,30,19},{1350,1153,2266}},
{0x371ccb5be99c125f,{63,37},{2745,4028}},
{0x3846d772484da48a,{71,29},{2868,2738}},
{0x385c406f07228ffa,{63,37},{3517,2291}},
{0x385fd6ac718a5477,{100},{617}},
{0x387bb90005c406d9,{100},{2803}},
{0x3894e5cd6474877f,{100},{1162}},
{0x38e04c532ad155ed,{63,37},{2745,4028}},
{0x38e083e59d3e373f,{100},{2291}},
{0x38eeab8e56ad437d,{100},{2022}},
{0x39af58bac404a79d,{100},{3321}},
{0x3a6f2c1532efb679,{63,37},{1487,1739}},
{0x3b8814362f2aef53,{56,23,21},{1892,2237,2291}},
{0x3c0db18f050636b8,{51,30,19},{1661,1770,2237}},
{0x3d0e5319a8f49401,{30,28,27,15},{2291,2673,2356,2226}},
{0x3d1cb2242a449c41,{63,37},{1153,1422}},
{0x3d3ea4458be3c013,{90},{1804}},
{0x3d499d94d4fade37,{90},{2868}},
{0x3d4d17f4656bbc36,{57,33,10},{2868,1978,2161}},
{0x3d88c3a59fb2c2b6,{100},{1153}},
{0x3db886ce10339f45,{90},{1350}},
{0x3ec8ba4d0f752393,{38,22,21,19},{1739,1804,1153,1422}},
{0x3f1105d9e0f40131,{100},{1162}},
{0x3f9d25cf4a8c4928,{41,24,22,13},{3063,2608,2745,3389}},
{0x3ff6a1e4c659aff3,{63,37},{901,2258}},
{0x4029908054b003e5,{47,28,25},{2226,2942,1978}},
{0x402ff34114a11134,{71,29},{1221,2331}},
{0x4035c9ecde00e323,{63,37},{3063,3322}},
{0x40989e97760684f8,{63,37},{4028,1699}},
{0x40b91efebffd04fa,{51,49},{1422,1739}},
{0x40c40b6ef621ee23,{100},{2291}},
{0x41102a7761d92d6e,{51,30,19},{1661,2291,2226}},
{0x413761a85be06fee,{44,26,16,14},{1674,1422,1804,1153}},
{0x4146e2c74bbe2ae7,{100},{578}},
{0x415337ff8cb71c4c,{63,37},{2226,2291}},
{0x41560a0d43d86c29,{63,37},{1350,1739}},
{0x420e12cedaaf98db,{47,28,25},{2868,2426,2226}},
{0x42252ac8d1ce0c86,{38,22,21,19},{2291,2237,1661,2745}},
{0x42a159022f042596,{47,28,25},{725,1609,277}},
{0x42a38a97cc375554,{51,49},{1153,1350}},
{0x4300bfdac94bd645,{51,30,19},{578,1292,1422}},
{0x439217bfd6cf51cb,{100},{3309}},
{0x4397ecc20890312a,{63,37},{2226,2486}},
{0x43a22c7dcefe1cd3,{63,37},{2426,3321}},
{0x43c5f1987971c8ed,{47,28,25},{2998,2803,2868}},
{0x441956af202d76a9,{100},{2745}},
{0x443556ae30a979ac,{100},{2285}},
{0x44372d08e4bc7738,{100},{2282}},
{0x444834712ca31395,{40,38,22},{3389,1827,1661}},
{0x44864a9e2b51788f,{90},{132}},
{0x451ae4e7daaa945c,{63,37},{1292,1350}},
{0x4523b8ba7f938400,{51,30,19},{3321,2356,2998}},
{0x45f0528a6e39b475,{90},{1762}},
{0x460aaf038d93ea13,{31,29,28,8,4},{1422,1739,1674,1097,1869}},
{0x465a78036ebbf1e0,{100},{1958}},
{0x46af9a9dbf56ff55,{100},{708}},
{0x47131f3f44776347,{40,38,22},{2356,2291,2942}},
{0x4781a350eaa94178,{100},{2332}},
{0x47c9396f9c2e7d78,{90},{1669}},
{0x484c1f3c887f6787,{100},{2218}},
{0x48821db8f209b164,{100},{2745}},
{0x48ff120e9aa70fc5,{90},{2942}},
{0x493477ec32a4a63d,{63,37},{1032,773}},
{0x4a21d3af5e0f8ad1,{63,37},{2803,2226}},
{0x4a6d300354059dcc,{100},{1803}},
{0x4a7d3cdfd3ea4f0d,{71,29},{2868,1762}},
{0x4acdf431bfb0cc33,{90},{1227}},
{0x4b576c9a76ddff67,{63,37},{1357,2267}},
{0x4b9495ff30d1c76f,{100},{2803}},
{0x4bd63d54075cda65,{63,37},{901,1674}},
{0x4c3d6cdbdb7fa797,{100},{2154}},
{0x4c5493ebfba17321,{47,28,25},{3517,2745,2942}},
{0x4c6aa231a0ae7a9a,{38,22,21,19},{1292,1357,705,1350}},
{0x4c76b6c3d3713d80,{71,29},{2803,2738}},
{0x4ca7c8b0d4e82ef8,{40,38,22},{2942,2291,2868}},
{0x4cc6fa4f929243f4,{100},{2130}},
{0x4d51eb948ec43822,{100},{3814}},
{0x4d627ecc688240bd,{51,30,19},{1978,2998,2347}},
{0x4d658ca93532348c,{90},{901}},
{0x4e0fe429e72ae0f6,{100},{1032}},
{0x4e1297011a588cd4,{90},{1350}},
{0x4e75e9ebc89ae457,{63,37},{1739,1153}},
{0x4e92e1601d31ea15,{44,26,16,14},{2226,3390,2673,3323}},
{0x4e9d67e1cee562ea,{90},{901}},
{0x4eda1d9cf4ca195d,{100},{3322}},
{0x4ef695f3ff557a07,{100},{2659}},
{0x4f2bf316a1e81ff3,{63,37},{1569,2721}},
{0x4f39f3f237dc4310,{36,34,19,11},{643,1292,1350,1032}},
{0x4f8ee7ea4913fb58,{100},{2332}},
{0x4fa707322e9b6e01,{33,32,18,17},{2868,2608,2745,2998}},
{0x4fba200c0978520a,{100},{2939}},
{0x5042c9588dae8adb,{51,30,19},{1739,1350,1804}},
{0x508eaba6d5e11a0a,{63,37},{2237,2942}},
{0x50d55eaafe0476ac,{100},{707}},
{0x514fc93b69fda04b,{90},{2291}},
{0x5224a8650e95ff8b,{44,26,24,6},{1350,1292,643,2258}},
{0x5240587022999167,{41,24,22,13},{2397,2332,773,2918}},
{0x52b72273922af62d,{100},{3517}},
{0x52edef322f35a256,{90},{707}},
{0x53329a46300495f8,{63,37},{3389,3063}},
{0x53a784fc481acfaa,{40,38,22},{2453,1227,1739}},
{0x53c361da31d2376f,{100},{1161}},
{0x53d6b8ff94338993,{100},{131}},
{0x53f3d4b9be44d834,{47,28,25},{1350,1292,1162}},
{0x5417cc10e4f00831,{100},{2325}},
{0x54ab0f9a7eecb0f4,{44,26,16,14},{2266,1350,1858,2434}},
{0x55023b13a0c89960,{37,22,20,12,9},{2942,3517,2226,2803,2421}},
{0x55fda1218606d9c7,{41,24,22,13},{2868,2226,2738,2998}},
{0x562b14493d334cc0,{47,28,25},{1739,1153,1422}},
{0x5701d0596f264805,{90},{4028}},
{0x570f07da718ae252,{90},{2803}},
{0x5773f3923d8d26da,{51,30,19},{1350,1487,1674}},
{0x577f5bc356e4d05d,{71,29},{2268,1350}},
{0x57efeddb1e4d89c5,{47,28,25},{1999,2462,391}},
{0x57fc78c65b934ea2,{90},{2998}},
{0x58655e17678a88e3,{100},{2356}},
{0x587d5b0bde6b83bf,{100},{1813}},
{0x58847addc0a10a13,{63,37},{3321,3322}},
{0x58a3dddef367d608,{63,37},{2426,2218}},
{0x58d30c5f09ba01de,{47,28,25},{3899,2745,2738}},
{0x58e9d7f8e7dc1ed7,{100},{4028}},
{0x58ea798be160b62f,{100},{2738}},
{0x5908cf4cd98a7f0d,{63,37},{1153,1350}},
{0x591b2f0aa0fc27fd,{90},{2998}},
{0x599a8b20649e3603,{100},{2874}},
{0x599d8f55bd057e2f,{38,22,21,19},{2117,1669,1153,1739}},
{0x59dfc023372abd31,{100},{2608}},
{0x59ef3c55b3b09eab,{90},{2998}},
{0x5a21e0164471aaad,{63,37},{2291,2942}},
{0x5a8bbe9e4249ea85,{71,29},{1242,794}},
{0x5aec261a621fe94f,{47,28,25},{2745,2868,2226}},
{0x5b547b5f88416fe4,{47,28,25},{2803,2868,2745}},
{0x5b6b8822f897d460,{41,24,22,13},{388,1282,2267,2331}},
{0x5ba0e46092d19f96,{71,29},{2226,2608}},
{0x5c6234e26f3acd30,{100},{388}},
{0x5c860b4e51433b6d,{26,25,14,13,12,10},{2942,2226,1827,2745,2608,3389}},
{0x5cee4efcaaee7ae4,{90},{901}},
{0x5d11683aec78e5e4,{100},{388}},
{0x5d5052fab8961271,{40,38,22},{1292,643,706}},
{0x5d93e9d876894905,{100},{1162}},
{0x5def1b174d2ad4e8,{90},{3517}},
{0x5e1a640f189434ab,{100},{2868}},
{0x5e765298aedf7bc8,{90},{4028}},
{0x5f103aa68e914876,{36,34,19,11},{1032,1422,1153,1292}},
{0x5f517a14863dbf03,{63,37},{2738,2608}},
{0x5f949af6c546ab66,{63,37},{774,1227}},
{0x5faba3c3a9309fdd,{38,22,21,19},{3389,2226,2745,2608}},
{0x5faf2414ccb8eecd,{71,29},{2332,1221}},
{0x5fcb7cd8c7a63d1c,{100},{2226}},
{0x5fda9f47a4f8e233,{71,29},{3321,1764}},
{0x5fe06b3870710787,{47,28,25},{2745,3321,2617}},
{0x5fe098c935f32667,{63,37},{2291,2356}},
{0x6052cf7f3aecb656,{56,23,21},{2285,2161,2218}},
{0x615c9571d399c528,{100},{2226}},
{0x61b3bab8f6d86b03,{47,28,25},{3517,2745,2942}},
{0x6202fe9d40c6c4f0,{41,24,22,13},{643,1153,705,1097}},
{0x6217e3996a35b18d,{100},{2868}},
{0x6267b2d1d3e9d4d1,{90},{4028}},
{0x62ff8fe085b4f714,{32,19,18,16,15},{2090,3690,3194,3063,3309}},
{0x63b37a5270e834de,{63,37},{2285,2284}},
{0x63dd14344aa550a7,{63,37},{1731,1162}},
{0x647bcfb37620117b,{100},{2426}},
{0x64bf715160314e8f,{100},{1661}},
{0x657914628f47a3e3,{90},{1739}},
{0x65a3951f00ef018e,{47,28,25},{1162,1609,1227}},
{0x65dceb347d23ff48,{47,28,25},{2745,3389,2608}},
{0x6816ba4561bcf32b,{100},{132}},
{0x6832ef21f1f3283d,{100},{2136}},
{0x68612ea1f11fe667,{100},{2942}},
{0x686b87aefb664402,{63,37},{2291,2998}},
{0x68c491e972ea9c22,{51,30,19},{2998,3321,1978}},
{0x695f48564153bef4,{47,28,25},{1934,1221,1429}},
{0x69663c4abb087585,{51,49},{3389,2608}},
{0x69d059e6c919e11f,{29,27,16,14,8,6},{2998,2868,2226,2803,2356,2738}},
{0x6a1fb7c1c40ebad3,{47,28,25},{2291,2738,2745}},
{0x6a44b2d95ecedba5,{35,33,32},{2998,2291,2868}},
{0x6a96a8d4363420f1,{100},{707}},
{0x6aafab6b7d022768,{56,23,21},{388,1869,1282}},
{0x6ade8fd35604bd9c,{100},{2331}},
{0x6b4801fb6ce433c7,{63,37},{1032,2137}},
{0x6b8774c9fb665804,{63,37},{2356,2738}},
{0x6c6d8c9e5cb598d6,{90},{2745}},
{0x6c843b0772c1f37e,{63,37},{1739,1669}},
{0x6ca5393794ed6887,{90},{3768}},
{0x6cd048fb569977cc,{57,33,10},{1804,1350,1674}},
{0x6ce116789e4d51f4,{90},{1347}},
{0x6cf703246ca0354c,{41,24,22,13},{2673,2291,1661,2226}},
{0x6d5e37adb2841cd8,{90},{1739}},
{0x6d8d57cef40a5cbd,{90},{2942}},
{0x6e1b6c7f34a9eb67,{63,37},{1869,1115}},
{0x6e244871f5e4f48f,{40,38,22},{2745,2608,4028}},
{0x6e916b3a874c8cb7,{63,37},{2745,2942}},
{0x6ea4789469f1891d,{63,37},{1422,1292}},
{0x6f0053141329a920,{100},{2291}},
{0x6f25c861d9764eb1,{100},{2268}},
{0x6f28815958470319,{63,37},{2683,3389}},
{0x6f5c6a597048138a,{100},{1674}},
{0x6f886e734411b7e5,{100},{2291}},
{0x6f89ecbe96ae2111,{51,30,19},{1221,643,2266}},
{0x6fad145def3a26de,{63,37},{2942,2803}},
{0x6fecab47873a6572,{63,37},{1804,1153}},
{0x702476150f6c6501,{90},{2356}},
{0x71336021369ca692,{100},{1153}},
{0x716d0ab34a86f228,{63,37},{2226,2868}},
{0x71d693bc81b15a13,{63,37},{2918,2267}},
{0x721a4f7bab677332,{71,29},{2942,2998}},
{0x72a7b3fdceb0279b,{63,37},{2332,2268}},
{0x72bb2ea8db831870,{90},{901}},
{0x72c9a54bbacd4347,{100},{1762}},
{0x73297e1b1272872c,{56,23,21},{1837,2803,1764}},
{0x73603110efd96ac1,{47,28,25},{1894,2022,1999}},
{0x73cc07d5f27b0bbd,{63,37},{1674,1292}},
{0x7420ed03bf3064c1,{90},{2803}},
{0x7441c23ca7a9d0c5,{63,37},{901,1674}},
{0x7462fd6cd8bee941,{51,30,19},{2434,1350,1858}},
{0x74697eec0dd5876d,{63,37},{1999,1669}},
{0x747aa606b07c660d,{38,22,21,19},{1669,1422,1869,1350}},
{0x74a3e76e38b49040,{100},{2738}},
{0x74af950f878ab6e6,{90},{388}},
{0x74f1c97e24088e40,{63,37},{1292,1162}},
{0x751cdd09053afbb4,{63,37},{2942,2745}},
{0x75bbbe5efcb028d4,{24,23,22,21,10},{2291,2356,2942,2226,2868}},
{0x75c2724dca390a4e,{41,24,22,13},{2942,2738,3063,2933}},
{0x7629ddb6b52dda5c,{100},{2291}},
{0x7635f25398c12e14,{51,49},{901,1153}},
{0x76afcf59900b482d,{47,28,25},{2998,2291,2226}},
{0x76b3520c853877c6,{90},{2745}},
{0x76bdde6942258959,{63,37},{1978,2738}},
{0x76eb5d1757c2dfee,{32,19,17,16,9,7},{1153,2918,1350,1292,705,1162}},
{0x7746136d58b3a27c,{51,49},{1097,131}},
{0x77498034df74f45e,{63,37},{2022,2918}},
{0x774bd23645453ce1,{37,22,20,12,9},{2942,2226,3389,2738,2426}},
{0x779a4c9f3cd694f3,{63,37},{261,771}},
{0x781a88728b7fc1e5,{100},{2426}},
{0x78e51240adb18142,{37,22,20,12,9},{2942,3517,2226,2356,2803}},
{0x78f52e0da5dc921b,{100},{1869}},
{0x79223816c7a6467d,{100},{1634}},
{0x7944b8eb75a0906d,{63,37},{1153,1350}},
{0x79f1da4f1642fc7f,{100},{3517}},
{0x79f50abea747119d,{90},{901}},
{0x7a10f96bb235fdb4,{32,19,18,16,15},{1292,64,1227,1804,1350}},
{0x7a328a00a5c46b29,{71,29},{2161,2745}},
{0x7aaa0f54479c17ee,{100},{1032}},
{0x7ae6f90a625f5027,{63,37},{4028,3321}},
{0x7b688066b0bd901d,{100},{1350}},
{0x7bbc28bb6a35ad4a,{90},{2803}},
{0x7bcc3d1a304a54e2,{38,22,21,19},{1739,1350,1804,1422}},
{0x7bebc47532acddc1,{56,23,21},{1422,1739,1350}},
{0x7c2907b3fdfd5072,{63,37},{2608,3322}},
{0x7c6dbeaa6b2ee9d4,{51,30,19},{1828,1892,2738}},
{0x7cebe482ca790ee4,{90},{3259}},
{0x7d58c738381df88b,{51,30,19},{1762,3517,2942}},
{0x7d6982f5d8980228,{63,37},{2942,2237}},
{0x7d8cbe80fed551f9,{63,37},{1350,1221}},
{0x7da5e00b80d792f9,{63,37},{1739,1153}},
{0x7df445dbcff8b28e,{47,28,25},{2942,2868,2226}},
{0x807950c95b11f2d7,{71,29},{2942,2868}},
{0x80b2fd3aa7d527c7,{100},{2291}},
{0x811fcb4d384492bb,{90},{4028}},
{0x812fbb321064ade9,{100},{2291}},
{0x81d2c031a59718e0,{51,28,21},{1153,1350,1422}},
{0x82462b0e3240225a,{90},{1674}},
{0x8282f189eb99a73d,{51,30,19},{578,1292,1422}},
{0x8287131ca04a2515,{90},{3517}},
{0x82fbef6b386ccd40,{51,30,19},{2942,3517,2803}},
{0x830b67c0f7ca0c4b,{90},{132}},
{0x8317742144be2321,{71,29},{1699,2813}},
{0x83507f93c6ea2777,{90},{1422}},
{0x83ef2764839fb07d,{38,36,14,12},{3517,2942,1978,3321}},
{0x840643fb3d0755ac,{100},{2083}},
{0x84427178bbf47b27,{90},{1357}},
{0x8492327a88f10b5b,{90},{2291}},
{0x85942857b0a2415c,{90},{388}},
{0x85c88314cad79c6a,{47,28,25},{1350,1153,1162}},
{0x85f8867aeeec30f4,{63,37},{1965,1837}},
{0x86016dfe64eae589,{100},{901}},
{0x8626eaafb53b05f0,{90},{2434}},
{0x86aa1e8b3a487402,{100},{2608}},
{0x86afd52faef9a3c6,{63,37},{1804,1350}},
{0x86d311bb5924f020,{90},{1350}},
{0x87463a13adc0b489,{47,28,25},{2745,3517,2942}},
{0x875236692733564e,{100},{2738}},
{0x8813afd40ef19bf5,{100},{1837}},
{0x883f5c397e4bf9e8,{47,28,25},{1674,1739,901}},
{0x8847cecda12ce20d,{63,37},{2803,1177}},
{0x890c0ecd8f41b1ff,{47,28,25},{2745,3321,2813}},
{0x8937833087afcb2e,{51,49},{2745,1661}},
{0x89565a25a748034a,{63,37},{1292,2434}},
{0x896c1e1daee86f81,{100},{901}},
{0x898af5e5b55cef97,{100},{1762}},
{0x8a7db35083aada3f,{71,29},{4028,1770}},
{0x8a980c3507ad475c,{35,20,19,17,9},{2673,2291,2745,2998,2868}},
{0x8b2d9d0aa341ea90,{100},{1739}},
{0x8b52e12d2479bb8b,{63,37},{1350,1804}},
{0x8b7b58e1ac6f1e5d,{63,37},{1539,1350}},
{0x8babb706e99cc352,{63,37},{901,1350}},
{0x8bcd48b710943ac4,{63,37},{2745,2942}},
{0x8c65111a798b4917,{63,37},{3390,3259}},
{0x8ca74f1438ff2ca8,{27,26,24,23},{2356,2291,2942,2226}},
{0x8d1806fbdbf22dbb,{63,37},{1350,901}},
{0x8db4156b062718e1,{51,30,19},{1292,1153,1539}},
{0x8db96728ab1646f9,{100},{1762}},
{0x8dd8346edcd5fe8d,{63,37},{2291,2942}},
{0x8df58a8efdc85079,{100},{1674}},
{0x8e9d3a319bdccd41,{41,24,22,13},{2942,2868,2608,2226}},
{0x8eb2a3dd310da436,{90},{2933}},
{0x8ef11b34412e2b2d,{47,28,25},{1739,1153,1422}},
{0x8f029292a8387c15,{47,28,25},{1153,1739,901}},
{0x8f15e77ee0f31786,{90},{1357}},
{0x8f582fbd9f0a02b9,{44,26,16,14},{2942,2868,2356,1699}},
{0x8fc91fc5d7abe465,{100},{773}},
{0x8fd8dea396cdc1c0,{100},{3517}},
{0x8fe333d117510e21,{100},{1161}},
{0x9053f45a07b05d3c,{47,28,25},{3389,2738,3321}},
{0x905a124febcf19a2,{100},{2282}},
{0x905e2dbeac8ad6c0,{51,49},{1544,1674}},
{0x90c072f8c3d9f19e,{51,49},{1350,1422}},
{0x9120a9a86b6635f5,{90},{1350}},
{0x9131fbdaf7d8bb81,{100},{3063}},
{0x91324369873db97c,{41,24,22,13},{2942,2356,2738,2868}},
{0x914c9905ebd2d1b8,{51,30,19},{3309,1837,1763}},
{0x914ebf1e1f1b5129,{47,28,25},{2738,1699,2868}},
{0x9192c3592d196960,{100},{2285}},
{0x91f37b7b1537be33,{39,23,14,13,11},{1674,1350,2434,1292,1858}},
{0x927647b155195ec9,{44,26,16,14},{2868,2998,2942,2738}},
{0x929e6c2ba70e23d0,{63,37},{2745,2803}},
{0x92ff39e5463ca982,{100},{1162}},
{0x939a637bc1ba0fd1,{51,30,19},{1292,1350,643}},
{0x93c58e2e4641c4de,{100},{1544}},
{0x93cea4cd5cc4ad1d,{25,24,23,22,6},{2998,2868,2356,2226,2738}},
{0x93e95da25e22243e,{36,34,19,11},{2942,2745,2803,2998}},
{0x93fc93b08bd5a5ab,{71,29},{4028,3063}},
{0x942fc5a698a3c43d,{56,23,21},{1739,1674,1153}},
{0x9461c1ac46d764c7,{100},{2738}},
{0x948bc9208514c584,{100},{2226}},
{0x94a8ddce28a88235,{63,37},{1350,1739}},
{0x94bee78bc53924a1,{100},{3389}},
{0x95d8929b82a48202,{38,36,14,12},{2942,1978,2426,2811}},
{0x9629e58722e993d4,{100},{3387}},
{0x96b189e7337919e8,{63,37},{3321,2745}},
{0x96f1bdc11f5f57a2,{63,37},{1350,1292}},
{0x9740ca86275950da,{51,30,19},{1153,1350,2266}},
{0x977770aa0333764c,{71,29},{388,2331}},
{0x9781f294b5535795,{41,24,22,13},{2942,1699,2738,2226}},
{0x979e7b60f6bba00b,{100},{3063}},
{0x9801188b1bc00bf8,{100},{2285}},
{0x981b8984dc02531a,{90},{2659}},
{0x983e0eca99ffe76d,{100},{2291}},
{0x99087c6d02d15190,{100},{2934}},
{0x9909b969af6c8e6f,{90},{388}},
{0x99f6c14b736a4f33,{41,24,22,13},{1162,705,771,1282}},
{0x9a00fc7cdd791abb,{71,29},{2745,4028}},
{0x9a18d6acc7e98acd,{71,29},{1153,1674}},
{0x9a3d6430b60cca81,{100},{1219}},
{0x9a79a5f22acfae2e,{100},{1032}},
{0x9aab81f9cb4e9e42,{90},{1153}},
{0x9acc838255e87bef,{100},{2803}},
{0x9ad3577c11340430,{51,30,19},{1227,1162,1739}},
{0x9b04efa2b096177a,{63,37},{1837,2683}},
{0x9b1a134198411dc1,{63,37},{2942,2745}},
{0x9b964f8fb186d12b,{100},{2738}},
{0x9badd6e04935585e,{100},{2426}},
{0x9bc79f73aac7790d,{47,28,25},{1153,1350,1162}},
{0x9bd4a1a6322cce87,{100},{729}},
{0x9cdb0be65ee5e817,{100},{2284}},
{0x9cdf016d3c7a9e4b,{63,37},{1739,2266}},
{0x9d42884793959bb1,{47,28,25},{3389,2745,2813}},
{0x9d825a73278a5d88,{63,37},{2813,3389}},
{0x9d912e6ced27f066,{71,29},{2022,1894}},
{0x9dadc378addd4084,{100},{1669}},
{0x9e043e571a9a4c56,{100},{1749}},
{0x9e35b2c6d4a2d46d,{63,37},{2932,2934}},
{0x9e3be0123c9fc842,{100},{1350}},
{0x9e79460ae9f5ae8b,{63,37},{2291,1762}},
{0x9e802625d55c617f,{51,30,19},{1350,128,2266}},
{0x9e8b1f66559fc95c,{100},{388}},
{0x9ead568efc85b1d2,{90},{901}},
{0x9ecbba358c91ca77,{100},{1033}},
{0x9f045ae44d5a23f5,{63,37},{2942,3517}},
{0x9fa8c6b16d0fc9b2,{100},{2486}},
{0x9fe5e082e0a3766c,{41,24,22,13},{1350,1357,773,1869}},
{0x9fedde545d826ae9,{100},{1683}},
{0xa03f1d3dd1ace6af,{63,37},{2942,2291}},
{0xa075a2a249e36647,{47,28,25},{3321,1699,2738}},
{0xa0793d47f3ada7e8,{63,37},{2434,2332}},
{0xa08b62f01723c32e,{51,30,19},{1292,1350,643}},
{0xa0aa5d24b523e78d,{100},{1764}},
{0xa0d28604aed1c5b3,{100},{2226}},
{0xa185108447fab20e,{71,29},{2683,2868}},
{0xa19629b40f88cf3b,{51,30,19},{1153,1292,2266}},
{0xa19757d8fa1c98da,{51,49},{3389,2608}},
{0xa1ac948ead66791e,{100},{2282}},
{0xa1b94c5c18e50160,{100},{2426}},
{0xa25ec0f93c16ca37,{100},{132}},
{0xa2bf06ee92731a9b,{71,29},{2942,2868}},
{0xa31632674c57330f,{51,30,19},{1739,1350,1804}},
{0xa350121d6e567248,{40,38,22},{2942,1661,1827}},
{0xa375625646509d67,{63,37},{1350,774}},
{0xa3ababbf66a75210,{56,23,21},{2285,1978,2299}},
{0xa3afe6402b2d4c20,{100},{4028}},
{0xa3d6918161409f66,{100},{3517}},
{0xa46a89a8cfcbb75b,{100},{3309}},
{0xa4b957065644fad5,{51,30,19},{1114,1739,1347}},
{0xa4f7530c88305a2f,{26,25,14,13,12,10},{2356,2226,2868,2738,2942,2803}},
{0xa4fe41b8c105b5a0,{63,37},{388,643}},
{0xa50c452adaab2fe9,{100},{1112}},
{0xa54f0e49626edc84,{71,29},{2868,2745}},
{0xa61329239b7041c2,{100},{2803}},
{0xa680ddd25919c112,{100},{1764}},
{0xa69ffe36a26b9b15,{90},{2998}},
{0xa6a456c50c1c95c3,{63,37},{706,1153}},
{0xa6ed4d36acde7b91,{71,29},{1739,1669}},
{0xa77f05bbb6d2ca59,{44,26,16,14},{1350,1669,2117,1282}},
{0xa7a59ee5421597fe,{90},{1282}},
{0xa7b23c66c0ec1c1f,{100},{3517}},
{0xa7de7c5615cb8f8f,{71,29},{2332,1153}},
{0xa7e5aadc3ac1f2e4,{44,26,24,6},{2291,2226,1661,2421}},
{0xa7f98539172d06ac,{35,20,19,17,9},{1350,1869,1153,1669,1739}},
{0xa8b92358f3d13ea6,{90},{2291}},
{0xa90824bd4238ffe1,{100},{388}},
{0xa930734d169e1967,{100},{2138}},
{0xa965211c31487b96,{71,29},{1350,705}},
{0xa977a6281b8c39ac,{63,37},{1292,1162}},
{0xa9d12fc3bfff107d,{90},{2738}},
{0xaa20c80480552e8c,{90},{388}},
{0xaa5c34731873c6d9,{41,24,22,13},{1153,1804,1350,1422}},
{0xaa693b065029d1ae,{47,28,25},{4028,2226,2291}},
{0xab28146948761c43,{90},{3321}},
{0xab4ed925a1154f39,{100},{2723}},
{0xac03e43b6f1ee75d,{100},{1669}},
{0xac076e5bde8f855c,{90},{2139}},
{0xac65cd126c710d7b,{100},{2285}},
{0xac6a073e881e4521,{100},{2161}},
{0xacb17485a219b78b,{63,37},{1739,1227}},
{0xad33f34f90bd4ac5,{90},{3517}},
{0xad45efb5bb0996de,{100},{2313}},
{0xad4cd9d120982342,{47,28,25},{1422,1858,2434}},
{0xad79cb9838531c36,{56,23,21},{2332,774,1032}},
{0xad83c7a7cade2c06,{100},{3389}},
{0xadc38e90e232b729,{47,28,25},{1227,1739,1153}},
{0xadc7e01083f1f393,{100},{3517}},
{0xae080e378ee6a85f,{51,30,19},{3517,2942,3321}},
{0xae0a92a5c1bfd04f,{90},{2291}},
{0xae4a50860cd47eeb,{63,37},{1674,1487}},
{0xae74caa3793bfbb9,{90},{3517}},
{0xaeab2b1e08b3fd6b,{90},{4028}},
{0xaeeaa1ca7fc96289,{51,30,19},{2942,2237,2745}},
{0xaf71ebf1e5674fcf,{63,37},{1153,1350}},
{0xaffe69c5c43a1fb4,{57,33,10},{2745,3389,2237}},
{0xb07782205cecd21c,{51,30,19},{2452,3028,66}},
{0xb0c209b77318aca1,{100},{1350}},
{0xb0d7c5513f406bbf,{47,28,25},{2803,2356,2868}},
{0xb0f94e4e7622ae95,{37,22,20,12,9},{1699,2738,2226,3389,3321}},
{0xb119b6ddcc72de02,{90},{901}},
{0xb1eda582df3e6754,{90},{901}},
{0xb25e8f3125f46f7e,{90},{901}},
{0xb2e373b740233bd7,{71,29},{3517,2356}},
{0xb307ec622e081c09,{100},{1097}},
{0xb31595ffefbb8309,{100},{2813}},
{0xb3486388e627166b,{63,37},{3069,2683}},
{0xb3670f76d825795d,{90},{1153}},
{0xb3bce356beb1cd60,{47,28,25},{2486,2291,2803}},
{0xb3ca4c25de38ac85,{71,29},{3387,3517}},
{0xb42abc2a72f1f425,{71,29},{2942,3389}},
{0xb49c19acf98d6f7c,{63,37},{1177,2073}},
{0xb5a8ea2e665d3b83,{47,28,25},{1544,259,454}},
{0xb5bb29506b10d4e9,{100},{1810}},
{0xb5de148cd39b7ac6,{51,30,19},{2738,1699,2868}},
{0xb5de85fff9ab0599,{51,30,19},{3517,2942,1762}},
{0xb64affb3444c407c,{47,28,25},{2868,2226,2426}},
{0xb6d8c9308eaf1451,{90},{3321}},
{0xb7039370ef0a2185,{43,41,16},{2908,2434,1221}},
{0xb72baad1589163cd,{63,37},{2745,2803}},
{0xb774609169606d35,{63,37},{2738,3389}},
{0xb78aa0c337a0c85c,{47,28,25},{1097,705,1858}},
{0xb78e67e185cad0f3,{63,37},{2813,3259}},
{0xb7cc013f355c696c,{90},{2803}},
{0xb827f80669ffb4cc,{38,22,21,19},{1153,1350,1739,1422}},
{0xb891a5356b7af290,{100},{1350}},
{0xb8b7767882b2b3d3,{47,28,25},{2356,2291,2942}},
{0xb90185fe21a7db89,{100},{2022}},
{0xb91eb96885abe406,{47,28,25},{2355,2152,1965}},
{0xb97f35b7e59c3a66,{63,37},{901,1153}},
{0xba108db719dc4a90,{90},{1292}},
{0xba6927705196da63,{100},{1252}},
{0xbaac6794be129faa,{51,30,19},{2738,2237,1371}},
{0xbb15228e08220ded,{100},{1739}},
{0xbb48f726d605202b,{38,22,21,19},{1153,2268,2332,705}},
{0xbb583214ad70a07d,{51,49},{3389,2738}},
{0xbb5e6bb7186eb545,{63,37},{1978,2426}},
{0xbb9085feb53d2219,{44,26,16,14},{2933,2811,1978,2813}},
{0xbbb9a04d1dafef50,{100},{578}},
{0xbbba7efc205c4362,{100},{2226}},
{0xbbce06a9b7f464af,{100},{1699}},
{0xbc4120a47be67405,{63,37},{2237,2291}},
{0xbc440eece9da18ee,{100},{3309}},
{0xbc8b1f3b1391cf15,{100},{2908}},
{0xbc8f08d750df2f8f,{100},{643}},
{0xbcc780fb661f2dd7,{47,28,25},{1153,1739,1422}},
{0xbcff5efd38054e82,{100},{578}},
{0xbe0e74f1535b3c1c,{51,49},{1828,2745}},
{0xbe298d9e51bdb53f,{31,30,29,10},{2226,2998,2868,2738}},
{0xbe817180932c9381,{100},{2981}},
{0xbe9449fb5b915a25,{35,30,15,10,6,4},{1804,1739,1350,1674,1609,1869}},
{0xbf1b66e43ddbf1fe,{56,23,21},{388,1153,1739}},
{0xbf3392d3cf5a050a,{63,37},{2745,2803}},
{0xbf821d66af3b5f08,{51,30,19},{1739,1422,1153}},
{0xbffee111371db75d,{90},{1350}},
{0xc03371e4015f4c0d,{100},{3366}},
{0xc05dcd8edcfe5c44,{63,37},{2942,2745}},
{0xc07f5a11b4f76b65,{47,28,25},{1153,1422,1739}},
{0xc0c2a697d1203fcc,{90},{3517}},
{0xc0ca38b5d7150777,{63,37},{2942,2745}},
{0xc16b921e0f041658,{33,32,18,17},{1739,1153,1804,1350}},
{0xc192ab886c71d770,{100},{901}},
{0xc194082c29ca56ff,{38,22,21,19},{1350,1292,1162,1153}},
{0xc19bad65f4712dc7,{100},{2868}},
{0xc1f2ae4fea1b821e,{44,26,16,14},{3387,3063,3322,3370}},
{0xc27266d865b51057,{43,41,16},{2788,1869,1350}},
{0xc2bd2776b431835f,{41,24,22,13},{2942,2868,2226,2426}},
{0xc2bde65d5a9a15c7,{51,30,19},{3321,2813,3389}},
{0xc2fddd50b871ef41,{90},{2434}},
{0xc316f2cef9285f65,{47,28,25},{1892,2237,1827}},
{0xc31a5a9f9241a9e2,{100},{3321}},
{0xc331d843a573d98b,{63,37},{1828,3901}},
{0xc3d134f7ee355c49,{100},{2285}},
{0xc3e317ef4aeed2d7,{63,37},{1350,1858}},
{0xc42b6055226205e3,{44,26,16,14},{2942,2226,2803,2356}},
{0xc44e6cad8a1f01a8,{90},{1292}},
{0xc4e702ab7a2d9532,{63,37},{1350,1153}},
{0xc5fea8ab6460c422,{41,24,22,13},{1739,388,1674,1227}},
{0xc6125f88348e7b39,{47,28,25},{1837,2226,2738}},
{0xc64d0168a15970b3,{47,28,25},{1661,2738,3389}},
{0xc75757a6e3d12140,{63,37},{2090,1770}},
{0xc77bf5db0da40d78,{100},{2226}},
{0xc79a7db41188570c,{100},{2942}},
{0xc7ba3b592d950a6c,{100},{388}},
{0xc7f08b6d97d8929c,{41,24,22,13},{2738,3321,2942,2868}},
{0xc8001cf9f1fe6521,{100},{2268}},
{0xc89be8da7bd1a4b7,{100},{1837}},
{0xc89ed31773b54cf0,{41,24,22,13},{128,643,1221,2266}},
{0xc8bc2850ee0ea87b,{90},{2346}},
{0xc8bde07f94293188,{100},{2998}},
{0xc8ee88da62b6bb00,{63,37},{2803,4028}},
{0xc91ddfde7592ba01,{90},{707}},
{0xc95e55ff0a0ac91f,{63,37},{1350,1032}},
{0xc9d66cb4c719a61d,{51,49},{3389,3321}},
{0xc9f74bc4ebf9231e,{100},{2096}},
{0xca1a88908234348f,{100},{1739}},
{0xca1ffb950113f4f0,{43,41,16},{1292,1350,2266}},
{0xcae9fbf4d1795963,{71,29},{2803,2942}},
{0xcb12e529430fcdea,{51,49},{901,1350}},
{0xcb18075a016c7f40,{100},{1162}},
{0xcbaf214c492322f0,{71,29},{2942,2421}},
{0xcbb590e5ef3d8731,{100},{3836}},
{0xcbc20ace06fc10e5,{63,37},{1350,1539}},
{0xccc852c8a862d9a1,{48,20,17,15},{1739,2266,1292,1350}},
{0xcd0d4744acb41659,{39,21,20,11,9},{1025,1292,1539,1422,1739}},
{0xce291a5d9c7e9de1,{71,29},{1739,2117}},
{0xcec55268e7f0abfb,{71,29},{1153,1350}},
{0xcece4db10bf1a2c3,{90},{1999}},
{0xced44334c59077ec,{51,30,19},{3194,2618,2226}},
{0xcf3dd252278de0dc,{56,23,21},{1350,1162,1153}},
{0xcf65dd49f57748f4,{63,37},{3321,2426}},
{0xcfbb720d3a74b90e,{90},{1661}},
{0xcfd56326eb830eef,{51,49},{2608,3194}},
{0xd008110841cb718c,{32,19,18,16,15},{2291,2942,2356,2868,2738}},
{0xd069ce5b4d211610,{100},{2942}},
{0xd070ffa28fe1f991,{100},{1350}},
{0xd0b66f5102284b0b,{100},{2268}},
{0xd1734dea75661b64,{47,28,25},{2683,3063,2426}},
{0xd17ba4757d4cbf62,{100},{1749}},
{0xd1f7ed31b5885f91,{30,28,27,15},{2868,2738,1763,1699}},
{0xd2c9543f6ce6ba79,{100},{1227}},
{0xd306c73dded62d0f,{100},{578}},
{0xd3213e52dc30a42c,{100},{578}},
{0xd36493c3e340e1b5,{63,37},{2802,2804}},
{0xd403d79056bfcba7,{90},{1153}},
{0xd4ba13da86ee13c1,{63,37},{1114,1242}},
{0xd4d0f3a86e840eec,{100},{2325}},
{0xd4f1379ad9ca0bc0,{100},{1350}},
{0xd53fa300ec14e311,{63,37},{1978,1827}},
{0xd54836b86306160c,{100},{1749}},
{0xd5e30cc0fe000fcc,{47,28,25},{1153,1422,1739}},
{0xd5f73cc9d2444534,{63,37},{705,2267}},
{0xd6ca239a63fbda6c,{35,33,32},{2942,2998,2745}},
{0xd738a0bb1fc0592d,{57,33,10},{1699,3389,2421}},
{0xd7bfb8d3afcb1f25,{100},{2291}},
{0xd8030b3c0f62f5a2,{63,37},{2226,2426}},
{0xd81df6a0274ee3b8,{41,24,22,13},{1221,1162,1869,1350}},
{0xd877140b3143dbc6,{100},{388}},
{0xd90bd7c1c74ecf85,{63,37},{1350,1674}},
{0xd9100a41b7124ffc,{71,29},{2325,2396}},
{0xd9521156286fd64c,{63,37},{1292,1422}},
{0xd9587b6a3ce0f14b,{90},{4028}},
{0xda7b3e0cb59403ec,{90},{2942}},
{0xdb0888e9dca51652,{51,30,19},{3517,2226,2803}},
{0xdb5e3b51acb80e66,{100},{2942}},
{0xdbdaceffcb17065b,{100},{901}},
{0xdc2b7f663f476fe7,{90},{2868}},
{0xdc86937c9dd264b5,{100},{707}},
{0xdcfcf711b9a50821,{41,24,22,13},{3389,1837,2161,2237}},
{0xdd807eedebf1fa8f,{47,28,25},{1674,1162,1221}},
{0xdde93b7fde9d7593,{100},{1739}},
{0xde09f33a85ec8721,{100},{4028}},
{0xdea9c0c45c3a5070,{63,37},{2291,3517}},
{0xdee9f4fd24ee356a,{29,27,16,14,8,6},{1934,773,1869,1282,1669,1422}},
{0xdef1d4174d93aa2e,{44,26,16,14},{1923,1350,1552,1999}},
{0xdf3560307faf5acc,{100},{261}},
{0xdf3fb44744b2bd55,{100},{3370}},
{0xdfcb9dbbac2bdf6c,{100},{1177}},
{0xe0130f0b1fb0e935,{100},{2942}},
{0xe02efb2e2b578f9f,{90},{1609}},
{0xe03bd6ca51cf8908,{100},{1739}},
{0xe0470ed6b9c09e83,{51,49},{1179,2291}},
{0xe05896bcc5fab98d,{100},{1869}},
{0xe0d087a25158ef9e,{100},{578}},
{0xe12f7b9ba51bc183,{63,37},{1097,1739}},
{0xe15d8c01ab0f96c4,{90},{1153}},
{0xe1c0ebd61570cd7f,{47,28,25},{2715,1292,667}},
{0xe1ef490ea23c12b5,{32,19,18,16,15},{2291,2998,2356,2226,2942}},
{0xe2344918d418d9aa,{44,26,16,14},{3321,3322,3389,2942}},
{0xe282f41af80e50d6,{63,37},{2868,1837}},
{0xe2ddaafa6dd95b5c,{63,37},{2738,3389}},
{0xe30ad626dd087ce3,{100},{2942}},
{0xe312f990cc345ca8,{63,37},{2299,2942}},
{0xe33a2051824b3c95,{47,28,25},{2117,1739,1422}},
{0xe3bc800e9fb26547,{63,37},{2942,2998}},
{0xe3ddaf31872bd143,{90},{1674}},
{0xe41ad309965b0251,{40,38,22},{2291,2356,2226}},
{0xe4256b42e1c3fdd5,{90},{1422}},
{0xe43145e7ca60be3f,{90},{3389}},
{0xe469f84c2045ddb4,{100},{1764}},
{0xe475d7a90da929fc,{36,34,19,11},{1739,1350,901,1153}},
{0xe47784fbe55219a8,{100},{1681}},
{0xe484bf029ddb7be4,{63,37},{4028,2608}},
{0xe510a0a76e711a0a,{51,49},{1739,1227}},
{0xe55d7e69c0ea7ef7,{63,37},{773,1032}},
{0xe57d30439bb6a089,{40,38,22},{2942,1764,3321}},
{0xe58df8e31a0abdba,{100},{1221}},
{0xe604ea0eecca1b48,{27,26,24,23},{2356,2291,2942,2226}},
{0xe6398ffdf38acfaa,{100},{2942}},
{0xe6541f19064d7529,{63,37},{1153,2331}},
{0xe70c44187c382875,{90},{2942}},
{0xe75106a8f5ea7aec,{100},{1187}},
{0xe75cf8f39052fc5c,{63,37},{2291,2942}},
{0xe7d8acd390c0fe58,{90},{901}},
{0xe7f8d2c60c6aecd2,{63,37},{1153,1292}},
{0xe80b3709887b8ff2,{100},{1892}},
{0xe8657d8ff13f3455,{90},{1091}},
{0xe8c8a4a657831685,{63,37},{2161,2868}},
{0xe9e5178d5119dec1,{100},{2942}},
{0xea245e4e20eacf2e,{63,37},{2291,4028}},
{0xea64bd18920c4a46,{63,37},{3517,2745}},
{0xeacc38d7cce20b61,{63,37},{1674,1221}},
{0xeb39a22d309f6137,{71,29},{1153,1292}},
{0xebfb55664ee358b5,{90},{1803}},
{0xec45b2cfde1b18c9,{100},{2268}},
{0xecd11548d20f1b46,{100},{2874}},
{0xed5973b3447fcfbb,{63,37},{3517,2942}},
{0xed987342b98e7347,{38,22,21,19},{1739,1422,1350,1804}},
{0xee53f3e5d55a6c31,{100},{1764}},
{0xee6e49e00ae89272,{47,28,25},{2998,2868,2803}},
{0xeec149b97e0a5f4f,{90},{2788}},
{0xef01296b1f711f25,{43,41,16},{2683,3322,3390}},
{0xef0ff7f389e6077d,{71,29},{901,1153}},
{0xef2109f095716c93,{47,28,25},{3259,3309,3194}},
{0xef826d789d6052f9,{71,29},{3517,2803}},
{0xefea96a0f86cc80c,{100},{773}},
{0xeffa7a4bbf769e39,{100},{1739}},
{0xf00d5f1dd89ded00,{63,37},{1674,261}},
{0xf085e88ad8d35811,{100},{1739}},
{0xf0ec25401323c157,{63,37},{388,1153}},
{0xf101112a5fbd27e2,{51,30,19},{1999,1350,1669}},
{0xf1333afb0f9f85d5,{35,33,32},{2942,2745,2608}},
{0xf173362118b05b7c,{51,30,19},{901,1674,1739}},
{0xf21a15a192645075,{100},{1153}},
{0xf325b261d52993ba,{63,37},{3389,1661}},
{0xf35910566040eae6,{100},{1869}},
{0xf3af53f1fcba1dcb,{71,29},{1739,1153}},
{0xf3b51bcdbc04f328,{90},{2998}},
{0xf40327446163926d,{100},{2161}},
{0xf467c65a9bc512b0,{100},{4028}},
{0xf4c9413fe031d863,{47,28,25},{2266,2434,643}},
{0xf500a83611d52cbb,{100},{2942}},
{0xf57147f5a4a711ca,{63,37},{2868,1837}},
{0xf5e3e678b9e6308b,{47,28,25},{2683,3063,2426}},
{0xf60e1bdc7c255be9,{63,37},{901,1350}},
{0xf629876c8c2ef91b,{90},{1153}},
{0xf64ef88acec3de81,{51,30,19},{901,1674,1739}},
{0xf6b16a7880a79847,{100},{2739}},
{0xf6dc679941eb4e40,{90},{388}},
{0xf73e4e70334b5012,{51,49},{2426,3063}},
{0xf74d14622ff8ef5d,{63,37},{2745,2738}},
{0xf7dc81277fe0e522,{63,37},{2332,2267}},
{0xf7fb69b56a2f734d,{71,29},{2942,3517}},
{0xf8042593f37917ee,{90},{1350}},
{0xf8855ffb3f4ad6f5,{100},{2283}},
{0xf88d2e8efc6e5060,{90},{1699}},
{0xf89eaa29c36cba74,{43,41,16},{1153,1350,1422}},
{0xf8b9e1f6f955f8f4,{51,30,19},{2745,2942,2868}},
{0xf90ba6f96a1da475,{100},{1177}},
{0xf935b8bbe9811371,{100},{2226}},
{0xf94333cea3197b6c,{100},{2918}},
{0xf9e4fec35bc6d385,{63,37},{1422,1350}},
{0xfa06f234669168ac,{63,37},{2868,2291}},
{0xfa210b5b6477e18f,{63,37},{1661,2803}},
{0xfb0a411654bb80ce,{63,37},{1674,2434}},
{0xfb2d0ac96e82c24e,{47,28,25},{2745,2942,2998}},
{0xfb5f7d6d4088b435,{100},{2745}},
{0xfb9d65807d0232af,{100},{1858}},
{0xfb9f091c6a8c38f1,{100},{2908}},
{0xfbb785735e976fd4,{47,28,25},{2291,2942,2421}},
{0xfbd80485cd21052a,{63,37},{1221,643}},
{0xfc93a0aec3e2de4d,{63,37},{2426,2226}},
{0xfca2c7b7bcffbd0c,{100},{2107}},
{0xfcad38e4dd26131a,{63,37},{1153,1350}},
{0xfd04348e6cf9813d,{40,38,22},{2291,2942,2421}},
{0xfd106e2b4ba5fe14,{100},{1827}},
{0xfd321c5f264b7ae6,{71,29},{1804,1350}},
{0xfd606bb92e2a9d73,{31,29,17,15,8},{771,1114,1544,2203,1242}},
{0xfe2d1bd4f102549d,{47,28,25},{1674,1350,2434}},
{0xff0525b5d8ae4441,{100},{1828}},
{0xff41487e2aa97f77,{63,37},{707,2203}},
{0xffd7bca2f8ffaead,{100},{2813}},
{0xffecb374a27a60e6,{100},{1382}},
}; //end of bk_def_book

//==================================================================

